﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

public class LJIntegrator : Integrator {

	public double [] Y;
	public double [] m;
	int n;
	int gnx = 15;
	int gny = 15;
	int gnz = 15;
	double gxmin = -10;
	double gxmax = 10;
	double gymin = -10;
	double gymax = 10;
	double gzmin = -10;
	double gzmax = 10;
	double dragCoefficient = 0.1;
	double heatCoefficient = .0;
	double LJsigma=1.0;
	double LJeps=10.0;
	double LJFMax=1000.0;
	System.Random random;

	public double drandRange(double min, double max) {
		double r = random.NextDouble ();
		return min + (max - min) * r;
	}

	public double LJF(double r) {
		double ratio = LJsigma / r;
		double ratio2 = ratio * ratio;
		double ratio6 = ratio2 * ratio2 * ratio2;
		double ratio12 = ratio6 * ratio6;
		double F =  48.0 * LJeps / r * (ratio12 - 0.5 * ratio6);
		return System.Math.Min (LJFMax, F);
	}
		
	public LJIntegrator(int n) {
		this.n = n;
		Init (6 * n);
		Y = new double[6 * n];
		m = new double[n];
		random = new System.Random();
		for (int i = 0; i < n; i++) {
			Y [6 * i + 0] = gxmin + drandRange (0.0f, 1.0f) * (gxmax - gxmin);
			Y [6 * i + 1] = gymin + drandRange (0.0f, 1.0f) * (gymax - gymin);
			Y [6 * i + 2] = gzmin + drandRange (0.0f, 1.0f) * (gzmax - gzmin);
			Y [6 * i + 3] = 0.0;
			Y [6 * i + 4] = 0.0;
			Y [6 * i + 5] = 0.0;
			m [i] = 1.0;
		}
	}

	public void forcePeriodic(double [] x) {
		for (int i = 0; i < n; i++) {
			while (x [6 * i + 0] < gxmin)
				x [6 * i + 0] += (gxmax - gxmin);
			while (x [6 * i + 0] >gxmax)
				x [6 * i + 0] -= (gxmax - gxmin);
			while (x [6 * i + 1] < gymin)
				x [6 * i + 1] += (gymax - gymin);
			while (x [6 * i + 1] >gymax)
				x [6 * i + 1] -= (gymax - gymin);
			while (x [6 * i + 2] < gzmin)
				x [6 * i + 2] += (gzmax - gzmin);
			while (x [6 * i + 2] >gzmax)
				x [6 * i + 2] -= (gzmax - gzmin);
		}
	}

	public override void RatesOfChange (double[] x, double[] xdot, double t)
	{
		RatesOfChange2 (x, xdot, t);
	}
	
	public void RatesOfChange1 (double[] x, double[] xdot, double t)
	{
		forcePeriodic (x);
		double grange = gxmax - gxmin;

		for (int i = 0; i < n; i++) {
			xdot [6 * i + 0] = x [6 * i + 3];
			xdot [6 * i + 1] = x [6 * i + 4];
			xdot [6 * i + 2] = x [6 * i + 5];
			xdot [6 * i + 3] = 0.0;
			xdot [6 * i + 4] = 0.0;
			xdot [6 * i + 5] = 0.0;
		}

		for (int i = 0; i < n; i++) {
			xdot [6 * i + 3] += -dragCoefficient * x [6 * i + 3];
			xdot [6 * i + 4] += -dragCoefficient * x [6 * i + 4];
			xdot [6 * i + 5] += -dragCoefficient * x [6 * i + 5];
			xdot [6 * i + 3] += heatCoefficient * drandRange (-1.0f, 1.0f);
			xdot [6 * i + 4] += heatCoefficient * drandRange (-1.0f, 1.0f);
			xdot [6 * i + 5] += heatCoefficient * drandRange (-1.0f, 1.0f);
		}

		for (int i = 0; i < n; i++) {
			int sixi = 6 * i;
			double xi = x [sixi + 0];
			double yi = x [sixi + 1];
			double zi = x [sixi + 2];
			int cxi = (int)((xi - gxmin) / (grange));
			int cyi = (int)((yi - gymin) / (grange));
			int czi = (int)((zi - gzmin) / (grange));
			double axi = 0;
			double ayi = 0;
			double azi = 0;
			for (int j = i + 1; j < n; j++) {
				int sixj = 6 * j;
				double xj = x [sixj + 0];
				double yj = x [sixj + 1];
				double zj = x [sixj + 2];
				int cxj = (int)((xj - gxmin) / (grange));
				int cyj = (int)((yj - gymin) / (grange));
				int czj = (int)((zj - gzmin) / (grange));
				double mi = m [i];
				for (int cxjn = cxj - 1; cxjn < cxj + 2; cxjn++) {
					int cxjnm = (cxjn + gnx) % gnx;
					for (int cyjn = cyj - 1; cyjn < cyj + 2 && cxjnm == cxi; cyjn++) {
						int cyjnm = (cyjn + gny) % gny;
						for (int czjn = czj - 1; czjn < czj + 2 && cyjnm == cyi; czjn++) {
							int czjnm = (czjn + gnz) % gnz;
							if (czjnm == czi) {
								double xjg = xj;
								double yjg = yj;
								double zjg = zj;
								if (cxjn - cxi > 2)
									xjg -= (gxmax - gxmin);
								if (cxjn - cxi < -2)
									xjg += (gxmax - gxmin);
								if (cyjn - cyi > 2)
									yjg -= (gymax - gymin);
								if (cyjn - cyi < -2)
									yjg += (gymax - gymin);
								if (czjn - czi > 2)
									zjg -= (gzmax - gzmin);
								if (czjn - czi < -2)
									zjg += (gzmax - gzmin);
								double dx = xjg - xi;
								double dy = yjg - yi;
								double dz = zjg - zi;
								double dr2 = dx * dx + dy * dy + dz * dz;
								double dr = System.Math.Sqrt (dr2);
								double Fodr = LJF (dr)/dr;
								double Fodri = Fodr / mi;
								double Fodrj = Fodr / m [j];
								axi -= Fodri * dx;
								ayi -= Fodri * dy;
								azi -= Fodri * dz;
								xdot [sixj + 3] += Fodrj * dx;
								xdot [sixj + 4] += Fodrj * dy;
								xdot [sixj + 5] += Fodrj * dz;
							}
						}
					}
				}
			}
			xdot [sixi + 3] += axi;
			xdot [sixi + 4] += ayi;
			xdot [sixi + 5] += azi;
		}

	}

	public int ghostmod(ref int cxgi, int offset) {
		int cxg = 0;
		if (cxgi < 0) {
			cxg = -1;
			cxgi += offset;
		} else if (cxgi >= offset) {
			cxg = 1;
			cxgi -= offset;
		}
		return cxg;
	}

	public void RatesOfChange2 (double[] x, double[] xdot, double t)
	{
		forcePeriodic (x);
		double grange = gxmax - gxmin;

		ArrayList[,,] cellPop = new ArrayList[gnx, gny, gnz];

		for (int i = 0; i < n; i++) {
			xdot [6 * i + 0] = x [6 * i + 3];
			xdot [6 * i + 1] = x [6 * i + 4];
			xdot [6 * i + 2] = x [6 * i + 5];
			xdot [6 * i + 3] = 0.0;
			xdot [6 * i + 4] = 0.0;
			xdot [6 * i + 5] = 0.0;
		}

		for (int i = 0; i < n; i++) {
			xdot [6 * i + 3] += -dragCoefficient * x [6 * i + 3];
			xdot [6 * i + 4] += -dragCoefficient * x [6 * i + 4];
			xdot [6 * i + 5] += -dragCoefficient * x [6 * i + 5];
			xdot [6 * i + 3] += heatCoefficient * drandRange (-1.0f, 1.0f);
			xdot [6 * i + 4] += heatCoefficient * drandRange (-1.0f, 1.0f);
			xdot [6 * i + 5] += heatCoefficient * drandRange (-1.0f, 1.0f);
		}

		for (int i = 0; i < n; i++) {
			int sixi = 6 * i;
			double xi = x [sixi + 0];
			double yi = x [sixi + 1];
			double zi = x [sixi + 2];
			int cxi = (int)((xi - gxmin) / (grange) * gnx);
			int cyi = (int)((yi - gymin) / (grange) * gny);
			int czi = (int)((zi - gzmin) / (grange) * gnz);
			if (cellPop [cxi, cyi, czi] == null) {
				cellPop [cxi, cyi, czi] = new ArrayList ();
			}
			cellPop [cxi, cyi, czi].Add (i);
		}

		for (int i = 0; i < n; i++) {
			int sixi = 6 * i;
			double xi = x [sixi + 0];
			double yi = x [sixi + 1];
			double zi = x [sixi + 2];

			int cxi = (int)((xi - gxmin) / (grange) * gnx);
			int cyi = (int)((yi - gymin) / (grange) * gny);
			int czi = (int)((zi - gzmin) / (grange) * gnz);
			double axi = 0;
			double ayi = 0;
			double azi = 0;
			double mi = m [i];
			for (int cxni = cxi - 1; cxni < cxi + 2; cxni++) {
				for (int cyni = cyi - 1; cyni < cyi + 2; cyni++) {
					for (int czni = czi - 1; czni < czi + 2; czni++) {

						int cxgi = cxni;
						int cygi = cyni;
						int czgi = czni;
						int cxg = ghostmod(ref cxgi, gnx);
						int cyg = ghostmod(ref cygi, gny);
						int czg = ghostmod(ref czgi, gnz);
						if (cellPop [cxgi, cygi, czgi] != null) {
							for (int cj = 0; cj < cellPop [cxgi, cygi, czgi].Count; cj++) {
								int j = (int)cellPop [cxgi, cygi, czgi] [cj];
								if (j > i) {
									int sixj = 6 * j;
									double xj = x [sixj + 0];
									double yj = x [sixj + 1];
									double zj = x [sixj + 2];
									double xjg = xj + cxg * grange;
									double yjg = yj + cyg * grange;
									double zjg = zj + czg * grange;
									double dx = xjg - xi;
									double dy = yjg - yi;
									double dz = zjg - zi;
									double dr2 = dx * dx + dy * dy + dz * dz;
									double dr = System.Math.Sqrt (dr2);
									double Fodr = LJF (dr) / dr;
									double Fodri = Fodr / mi;
									double Fodrj = Fodr / m [j];
									axi -= Fodri * dx;
									ayi -= Fodri * dy;
									azi -= Fodri * dz;
									xdot [sixj + 3] += Fodrj * dx;
									xdot [sixj + 4] += Fodrj * dy;
									xdot [sixj + 5] += Fodrj * dz;
								}
							}
						}
					}
				}
			}

			xdot [sixi + 3] += axi;
			xdot [sixi + 4] += ayi;
			xdot [sixi + 5] += azi;
		}

	}
}
